vec2 complexMultiply(vec2 a, vec2 b)
{
    return vec2(a.x * b.x - a.y * b.y, a.x * b.y + a.y * b.x);
}

float mandelbrot(vec2 z, vec2 c)
{
    const int iterations = 500;

    for (int i = 0; i < iterations; i++)
    {
        z = complexMultiply(z, z) + c;
        if (length(z) > 2.0)
            return float(i) / float(iterations);
    }

    // Pro vnitřek množiny vracíme -1,
    // abychom snadno pomocí znaménka rozlišili
    // vnitřek od zabarveného okraje
    return -1.0;
}

vec2 mapToComplex(vec2 uv)
{
    vec2 complex = uv * 2.0; // ručně odzoomujeme
    
    // Je libo zrcadlení? Odkomentujte tento řádek!
    //complex.x = abs(complex.x);
    
    const float pi = 3.14159265;
    // Perioda animace v sekundách
    const float period = 20.0;
    // Minimální úroveň zoomu
    // pozor že v prvním řádku funkce už prakticky odzoomujeme
    const float minzoom = 2.0;
    // Maximální úroveň zoomu
    const float maxzoom = 12.0;
    
    // Periodicitu animace zařídíme pomocí sinu
    float zoom = (sin(iTime*2.0*pi/period)*0.5+0.5);
    
    zoom = zoom*(maxzoom - minzoom) + minzoom;
    
    // Hezké zoomování zařídí dělení hodnotou 2^zoom
    // complex dělíme (zmenšujeme), protože menší rozsah
    // znamená, že vidíme menší část fraktálu, tedy větší zoom
    complex /= pow(2.0, zoom);
    
    complex += vec2(-0.745428, -0.113009);
    
    return complex;
}

vec3 colorFractal(float v)
{
    if(v < 0.0)
        return vec3(0.0);
    v *= 3.0;
    v += 6.8;
    return vec3(sin(v), cos(v), sin(v * 0.7 + 0.85)) * 0.5 + 0.5;
}

void mainImage(out vec4 fragColor, in vec2 fragCoord)
{
    vec2 uv = fragCoord/iResolution.xy;

    uv = uv * 2.0 - 1.0; // Převedeme uv do rozsahu -1..1
    
	// "Natáhneme" X tak, aby byl zachován poměr stran
    uv.x *= iResolution.x / iResolution.y;

    // Nyní je tedy čtverec v souřadnicích i čtvercem na monitoru

    vec2 complex = mapToComplex(uv);

    // Použijeme naši funkci pro zabarvení
    vec3 col = colorFractal(mandelbrot(vec2(0.0), complex));

    fragColor = vec4(col, 1.0);
}
