vec2 complexMultiply(vec2 a, vec2 b)
{
    return vec2(a.x * b.x - a.y * b.y, a.x * b.y + a.y * b.x);
}

// Ve výchozím stavu nepoužitá funkce
// s natvrdo zadrátovanou 4. mocninou
float mandelbrot(vec2 z, vec2 c)
{
    const int iterations = 500;

    for (int i = 0; i < iterations; i++)
    {
        // z = z^4+c
        z = complexMultiply(z, z);
        z = complexMultiply(z, z) + c;
        if (length(z) > 2.0)
            return float(i) / float(iterations);
    }

    // Pro vnitřek množiny vracíme -1,
    // abychom snadno pomocí znaménka rozlišili
    // vnitřek od zabarveného okraje
    return -1.0;
}

// Obecná komplexní mocnina
vec2 complexPower(vec2 c, float p)
{
    float l = length(c);
    
    if(l < 0.00001)
        return vec2(0.0);
    
    c = normalize(c);
    
    float angle = acos(c.x);
    
    if(c.y < 0.0)
        angle = -angle;
    
    angle *= p;
    
    return vec2(cos(angle), sin(angle)) * pow(l, p);
}

// funkce s obecnou mocninou
float mandelbrotPower(vec2 z, vec2 c, float power)
{
    const int iterations = 500;

    for (int i = 0; i < iterations; i++)
    {
        z = complexPower(z, power) + c;
        if (length(z) > 2.0)
            return float(i) / float(iterations);
    }

    // Pro vnitřek množiny vracíme -1,
    // abychom snadno pomocí znaménka rozlišili
    // vnitřek od zabarveného okraje
    return -1.0;
}

vec2 mapToComplex(vec2 uv)
{
    vec2 complex = uv * 2.0;
    return complex;
}

vec3 colorFractal(float v)
{
    return vec3(pow(v*2.5, 0.5));
}

void mainImage(out vec4 fragColor, in vec2 fragCoord)
{
    vec2 uv = fragCoord/iResolution.xy;

    uv = uv * 2.0 - 1.0; // Převedeme uv do rozsahu -1..1
    
	// "Natáhneme" X tak, aby byl zachován poměr stran
    uv.x *= iResolution.x / iResolution.y;

    // Nyní je tedy čtverec v souřadnicích i čtvercem na monitoru

    vec2 complex = mapToComplex(uv);

    // animujeme mocninu v čase
    float power = (sin(iTime * 0.5) * 0.5 + 0.5) * 4.0 + 1.0;
    
    // Použijeme naši funkci pro zabarvení
    vec3 col = colorFractal(mandelbrotPower(vec2( 0.0), complex, power));

    fragColor = vec4(col, 1.0);
}
