vec2 complexMultiply(vec2 a, vec2 b)
{
    return vec2(a.x * b.x - a.y * b.y, a.x * b.y + a.y * b.x);
}

float mandelbrot(vec2 z, vec2 c)
{
    const int iterations = 500;

    for (int i = 0; i < iterations; i++)
    {
        z = complexMultiply(z, z) + c;
        if (length(z) > 2.0)
            return float(i) / float(iterations);
    }

    // Pro vnitřek množiny vracíme -1,
    // abychom snadno pomocí znaménka rozlišili
    // vnitřek od zabarveného okraje
    return -1.0;
}

vec2 mapToComplex(vec2 uv)
{
    vec2 complex = uv * 1.3; // ručně odzoomujeme
    return complex;
}

vec3 colorFractal(float v)
{
    if(v < 0.0)
        return vec3(0.0);
    return vec3(sin(v * 8.5 + 4.3), sin(v * 8.0 + 3.6), sin(v * 6.0 + 3.8)) * 0.5 + 0.5;
}

void mainImage(out vec4 fragColor, in vec2 fragCoord)
{
    vec2 uv = fragCoord/iResolution.xy;

    uv = uv * 2.0 - 1.0; // Převedeme uv do rozsahu -1..1
    
	// "Natáhneme" X tak, aby byl zachován poměr stran
    uv.x *= iResolution.x / iResolution.y;

    // Nyní je tedy čtverec v souřadnicích i čtvercem na monitoru

    vec2 complex = mapToComplex(uv);

    float angle = iTime;
    // k bodu 0-0.65i přičítáme bod na kružnici o poloměru 0.02,
    // který kružnici obkrouží každých 2pi sekund
    vec2 c = vec2(sin(iTime), cos(iTime)) * 0.02 + vec2(0.0, -0.65);
    
    // Použijeme naši funkci pro zabarvení
    vec3 col = colorFractal(mandelbrot(complex, c));

    fragColor = vec4(col, 1.0);
}
