vec2 random2(vec2 st)
{
    vec2 s = vec2(dot(st, vec2(12.3456, 34.1415)),
                  dot(st, vec2(42.2154, 15.2854)));
    return fract(sin(s) * 45678.9) * 2.0 - 1.0;
}

float gradientNoise(vec2 st)
{
    vec2 cell = floor(st);
    vec2 f = fract(st);

    vec2 s00 = random2(cell);
    vec2 s01 = random2(cell + vec2(0, 1));
    vec2 s10 = random2(cell + vec2(1, 0));
    vec2 s11 = random2(cell + vec2(1, 1));

    float d00 = dot(s00, f);
    float d01 = dot(s01, f - vec2(0, 1));
    float d10 = dot(s10, f - vec2(1, 0));
    float d11 = dot(s11, f - vec2(1, 1));

    vec2 u = smoothstep(0.0, 1.0, f);

    float noise = mix(mix(d00, d10, u.x), mix(d01, d11, u.x), u.y);
    return noise * 0.5 + 0.5;
}

// "Barevný" gradient noise (tři hodnoty v -1..1)
vec3 gradientNoise3(vec2 st)
{
    // Souřadnice šumů pro různé složky jsou různé,
    // aby byl výsledný šum barevný
    vec3 result = vec3(
        gradientNoise(st),
        gradientNoise(st + vec2(1.11, 2.22)),
        gradientNoise(st + vec2(3.33, 4.44))
    );
    // Hodnoty převedeme do rozsahu -1..1
    result = result * 2.0 - 1.0;
    // Hodnoty také něčím vynásobíme, aby byly více kontrastní
    result = clamp(result * 1.7, -1.0, 1.0);
    return result;
}

// "Barevný" fbm (tři hodnoty v -1..1)
vec3 fbm3(vec2 st)
{
    vec3 val = vec3(0.0);

    float p = 0.5;

    const float angle = 1.0;

    mat2 rotation = mat2(
        cos(angle), sin(angle),
       -sin(angle), cos(angle)
    );

    for (int i = 0; i < 5; i++)
    {
        val += gradientNoise3(st) * p;
        p *= 0.5;
        st *= 2.0;

        st += vec2(1.181, 0.57);
        st = rotation * st;
    }

    return val;
}

// Více fbm zapojených do sebe a animovaných
vec3 multiFbm(vec2 st)
{
    float ellapsed = iTime;
    vec2 uv1 = st + ellapsed * vec2(0.05, 0.1);
    vec2 uv2 = st + ellapsed * vec2(0.1, -0.1);
    vec2 uv3 = st + ellapsed * vec2(-0.05, 0.05);
    return fbm3(uv1 + fbm3(uv2 + fbm3(uv3).xy * 0.7).xy * 0.5);
}

void mainImage(out vec4 fragColor, in vec2 fragCoord)
{
    vec2 uv = fragCoord/iResolution.xy;
    // Střed souřadnic chceme uprostřed obrazu
    uv = uv * 2.0 - 1.0;
    // Korekce pro správný poměr stran
    uv.x *= iResolution.x / iResolution.y;

    // Hodnoty převedené z -1..1 zpět do 0..1
    vec3 col = multiFbm(uv) * 0.5 + 0.5;

    fragColor = vec4(col, 1.0);
}

